import os

import pandas as pd
import pingouin as pg

from config import Config, DataConfig


def answer_ratings_conversion() -> pd.DataFrame:
    input_df = pd.read_csv(
        f"{Config.InputPath}/answer_ratings.csv",
        delimiter=","
    )
    df = pd.melt(
        frame=input_df,
        id_vars=["PID", "Survey ID", "VL-Bar", "VL-Scatter"],
        value_name="Answer"
    )

    df["ChartType"] = df.apply(
        axis=1,
        func=lambda row: "Bar" if int(row["variable"]) in DataConfig.TaskMap["Bar"] else "Scatter"
    )
    df["Adaptation"] = df.apply(
        axis=1,
        func=lambda row: "Adapted" if int(row["variable"]) in DataConfig.TaskMap["Adapted"] else "Non-Adapted"
    )
    df["TaskNr"] = df.apply(
        axis=1,
        func=lambda row: int(row["variable"])
    )
    df["TaskGroup"] = df.apply(
        axis=1,
        func=lambda row: int((row["TaskNr"] - 1) % 5) + 1
    )

    df.pop("variable")
    df.to_csv(f"{Config.DataOutputPath}/answer_ratings.csv")

    return df


def answer_times_conversion() -> pd.DataFrame:
    input_df = pd.read_csv(
        f"{Config.InputPath}/answer_times.csv",
        delimiter=","
    )
    df = pd.melt(
        frame=input_df,
        id_vars=["PID", "Survey ID", "VL-Bar", "VL-Scatter"],
        value_vars=[str(i) for i in range(1, 21)],
        value_name="Time"
    )

    df["ChartType"] = df.apply(
        axis=1,
        func=lambda row: "Bar" if int(row["variable"]) in DataConfig.TaskMap["Bar"] else "Scatter"
    )
    df["Adaptation"] = df.apply(
        axis=1,
        func=lambda row: "Adapted" if int(row["variable"]) in DataConfig.TaskMap["Adapted"] else "Non-Adapted"
    )
    df["TaskNr"] = df.apply(
        axis=1,
        func=lambda row: int(row["variable"])
    )
    df["TaskGroup"] = df.apply(
        axis=1,
        func=lambda row: int((row["TaskNr"] - 1) % 5) + 1
    )

    # Calculate the trimmed value
    mean = df["Time"].mean()
    error = df["Time"].std()
    trimmed_value = float(mean + 2 * error)
    # Count how many values will be trimmed
    trimmed_count = 0
    for row in df.iterrows():
        if row[1]["Time"] > trimmed_value:
            trimmed_count += 1
    print(f"Total trimmed values: ${100 * float(trimmed_count) / df.shape[0]}")
    # Trim the data values for the time since we have several outlier
    df["Time"] = df.apply(axis=1,
        func=lambda row: row["Time"] if row["Time"] <= trimmed_value else trimmed_value
    )

    df.pop("variable")
    df.to_csv(f"{Config.DataOutputPath}/answer_times.csv")

    return df


def ueq_conversion() -> pd.DataFrame:
    input_df = pd.read_csv(
        f"{Config.InputPath}/ueq.csv",
        delimiter=","
    )
    df = pd.melt(
        frame=input_df,
        id_vars=["PID", "Survey ID", "VL-Bar", "VL-Scatter"],
        value_name="Rating"
    )

    df["ChartType"] = df.apply(
        axis=1,
        func=lambda row: "Bar" if "B" == row["variable"][0] else "Scatter"
    )
    df["Adaptation"] = df.apply(
        axis=1,
        func=lambda row: "Adapted" if "A" == row["variable"][2] else "Non-Adapted"
    )
    df["QuestionNr"] = df.apply(
        axis=1,
        func=lambda row: int(row["variable"][-1])
    )
    ux_map = {"D": "Dependability", "U": "Usefulness", "I": "Intuitive Use"}
    df["QuestionGroup"] = df.apply(
        axis=1,
        func=lambda row: ux_map[row["variable"][-3]]
    )

    df.pop("variable")
    df.to_csv(f"{Config.DataOutputPath}/ueq.csv")

    return df


def familarity_conversion() -> pd.DataFrame:
    input_df = pd.read_csv(
        f"{Config.InputPath}/familiarity.csv",
        delimiter=","
    )

    df = pd.DataFrame()
    for key in ["PID", "VL-Bar", "VL-Scatter", "Pie Chart", "Bar Chart", "Line Chart", "Area Chart", "Scatterplot", "Radar Chart", "Treemap", "Network Graph"]:
        df[key] = input_df[key]

    df.to_csv(f"{Config.DataOutputPath}/familiarity.csv")

    return df


def preference_conversion() -> pd.DataFrame:
    input_df = pd.read_csv(
        f"{Config.InputPath}/preference.csv",
        delimiter=","
    )
    input_df.to_csv(f"{Config.DataOutputPath}/preference.csv")

    return input_df


if __name__ == '__main__':
    answer_ratings_conversion()
    answer_times_conversion()
    ueq_conversion()
    familarity_conversion()
    preference_conversion()
